#pragma once
/*
 *  Copyright (C) 2024  Brett Terpstra
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef BLT_WITH_GRAPHICS_FONT_H
#define BLT_WITH_GRAPHICS_FONT_H

#include <ft2build.h>
#include FT_FREETYPE_H
#include <string>
#include <string_view>
#include <memory>
#include <optional>
#include "blt/std/types.h"
#include <blt/math/vectors.h>

namespace blt::gfx::font
{
    extern const unsigned int default_font_compressed_data[];
    extern const unsigned int default_font_compressed_size;
    
    FT_Library* create();
    
    void cleanup(FT_Library* lib = nullptr);
    
    class font_face_t
    {
        public:
            explicit font_face_t(std::string_view path, std::string_view font_name);
            
            explicit font_face_t(std::string_view font_name, const blt::u8* data, blt::size_t size, bool compressed = false);
            
            void set_pixel_sizes(blt::u32 width, blt::u32 height) const;
            
            [[nodiscard]] std::optional<int> load_char(FT_ULong c) const;
            
            [[nodiscard]] FT_Face* get() const
            { return face.get(); }
            
            [[nodiscard]] std::string_view get_name() const
            { return font_name; }
        
        private:
            // decompressed data ownership
            std::shared_ptr<blt::u8[]> internal_uncompressed_data;
            std::shared_ptr<FT_Face> face;
            std::string font_name;
    };
    
    struct font_file_t
    {
        font_face_t font;
        blt::u64 character_min_index;
        blt::u64 character_max_index;
        
        font_file_t(font_face_t font, u64 characterMinIndex, u64 characterMaxIndex);
    };
    
    struct glyph_t
    {
        blt::vec2ui size;
        blt::vec2ui bearing;
        blt::i64 advance;
        
        glyph_t(const vec2ui& size, const vec2ui& bearing, blt::i64 advance);
    };
    
}

#endif //BLT_WITH_GRAPHICS_FONT_H
